@@ -1,12 +1,20 @@
+import sys
 from io import StringIO
 
 try:
     from pathlib import Path
 except ImportError:
     from pathlib2 import Path
-from readme_renderer.rst import render
+
+import pytest
+
+if sys.version_info < (3, 9):
+    from readme_renderer.rst import render
+else:
+    render = None
 
 
+@pytest.mark.skipif(render is None, reason="html5lib issue #419")
 def test_pypi_rendering():
     # Adapted from `https://stackoverflow.com/questions/46766570/`.
     readme = Path("README.rst")
