@@ -143,11 +143,15 @@ class TestDTLSCookie:
         assert cookie.timeout == 1000
 
 
-class _TestBaseConfiguration(Chain):
+class _BaseConfiguration(Chain):
     @pytest.fixture
     def conf(self):
         raise NotImplementedError
 
+    @pytest.fixture
+    def version(self):
+        raise NotImplementedError
+
     @pytest.mark.parametrize("validate", [True, False])
     def test_set_validate_certificates(self, conf, validate):
         conf_ = conf.update(validate_certificates=validate)
@@ -162,7 +166,7 @@ class _TestBaseConfiguration(Chain):
         conf_ = conf.update(certificate_chain=chain)
         assert conf_.certificate_chain == chain
 
-    @pytest.mark.parametrize("ciphers", [ciphers_available()])
+    @pytest.mark.parametrize("ciphers", (ciphers_available(),))
     def test_set_ciphers(self, conf, ciphers):
         conf_ = conf.update(ciphers=ciphers)
         assert conf_.ciphers == ciphers
@@ -177,6 +181,14 @@ class _TestBaseConfiguration(Chain):
             NextProtocol(_) for _ in inner_protocols
         )
 
+    def test_lowest_supported_version(self, conf, version):
+        conf_ = conf.update(lowest_supported_version=version)
+        assert conf_.lowest_supported_version is version
+
+    def test_highest_supported_version(self, conf, version):
+        conf_ = conf.update(highest_supported_version=version)
+        assert conf_.highest_supported_version is version
+
     @pytest.mark.parametrize("store", [TrustStore.system()])
     def test_trust_store(self, conf, store):
         conf_ = conf.update(trust_store=store)
@@ -187,37 +199,39 @@ class _TestBaseConfiguration(Chain):
     def test_set_sni_callback(self, conf, callback):
         assert conf.sni_callback is None
 
+    @pytest.mark.parametrize("psk", [None, ("client", b"the secret key")])
+    def test_psk(self, conf, psk):
+        assert conf.pre_shared_key is None
+        conf_ = conf.update(pre_shared_key=psk)
+        assert conf_.pre_shared_key == psk
 
-class TestTLSConfiguration(_TestBaseConfiguration):
+    @pytest.mark.parametrize(
+        "psk_store", [None, {"client": b"the secret key"}]
+    )
+    def test_psk_store(self, conf, psk_store):
+        assert conf.pre_shared_key_store is None
+        conf_ = conf.update(pre_shared_key_store=psk_store)
+        assert conf_.pre_shared_key_store == psk_store
+
+
+class TestTLSConfiguration(_BaseConfiguration):
     @pytest.fixture
     def conf(self):
         return TLSConfiguration()
 
-    @pytest.mark.parametrize("version", TLSVersion)
-    def test_lowest_supported_version(self, conf, version):
-        conf_ = conf.update(lowest_supported_version=version)
-        assert conf_.lowest_supported_version is version
-
-    @pytest.mark.parametrize("version", TLSVersion)
-    def test_highest_supported_version(self, conf, version):
-        conf_ = conf.update(highest_supported_version=version)
-        assert conf_.highest_supported_version is version
+    @pytest.fixture(params=TLSVersion)
+    def version(self, request):
+        return request.param
 
 
-class TestDTLSConfiguration(_TestBaseConfiguration):
+class TestDTLSConfiguration(_BaseConfiguration):
     @pytest.fixture
     def conf(self):
         return DTLSConfiguration()
 
-    @pytest.mark.parametrize("version", DTLSVersion)
-    def test_lowest_supported_version(self, conf, version):
-        conf_ = conf.update(lowest_supported_version=version)
-        assert conf_.lowest_supported_version is version
-
-    @pytest.mark.parametrize("version", DTLSVersion)
-    def test_highest_supported_version(self, conf, version):
-        conf_ = conf.update(highest_supported_version=version)
-        assert conf_.highest_supported_version is version
+    @pytest.fixture(params=DTLSVersion)
+    def version(self, request):
+        return request.param
 
     @pytest.mark.parametrize("anti_replay", [True, False])
     def test_set_anti_replay(self, conf, anti_replay):
@@ -289,13 +303,31 @@ class TestServerContext(TestBaseContext):
         assert isinstance(context.wrap_buffers(), TLSWrappedBuffer)
 
 
-class _TestCommunicationBase(Chain):
-    CLOSE_MESSAGE = b"bye"
+class _CommunicationBase(Chain):
+    @pytest.fixture(scope="class")
+    def proto(self):
+        raise NotImplementedError
 
     @pytest.fixture(scope="class")
     def version(self):
         raise NotImplementedError
 
+    @pytest.fixture(scope="class")
+    def ciphers(self):
+        return None
+
+    @pytest.fixture(scope="class")
+    def srv_hostname(self):
+        return "End Entity"
+
+    @pytest.fixture(scope="class")
+    def cli_psk(self):
+        return None
+
+    @pytest.fixture(scope="class")
+    def srv_psk(self):
+        return None
+
     @pytest.fixture(scope="class")
     def srv_conf(self):
         raise NotImplementedError
@@ -304,10 +336,6 @@ class _TestCommunicationBase(Chain):
     def cli_conf(self):
         raise NotImplementedError
 
-    @pytest.fixture
-    def step(self):
-        raise NotImplementedError
-
     def echo(self, sock):
         raise NotImplementedError
 
@@ -328,49 +356,63 @@ class _TestCommunicationBase(Chain):
         return store
 
     @pytest.fixture
-    def server(self, srv_conf, version):
+    def server(self, srv_conf, version, proto):
         ctx = ServerContext(srv_conf)
-        sock = ctx.wrap_socket(socket.socket(socket.AF_INET, self.proto))
+        sock = ctx.wrap_socket(socket.socket(socket.AF_INET, proto))
         sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
         sock.bind(("", 0))
-        if self.proto == socket.SOCK_STREAM:
+        if proto == socket.SOCK_STREAM:
             sock.listen(1)
 
         runner = mp.Process(target=self.echo, args=(sock,))
         runner.start()
         yield sock
-        runner.join(0.1)
+        runner.terminate()
+        runner.join()
         with suppress(OSError):
             sock.close()
-        runner.terminate()
 
     @pytest.fixture
-    def client(self, server, cli_conf):
+    def client(self, server, srv_hostname, cli_conf, proto):
         ctx = ClientContext(cli_conf)
         sock = ctx.wrap_socket(
-            socket.socket(socket.AF_INET, self.proto),
-            server_hostname="End Entity",
+            socket.socket(socket.AF_INET, proto), server_hostname=srv_hostname,
         )
         sock.connect(server.getsockname())
-        block(sock.do_handshake)
         yield sock
-        with suppress(OSError):
-            block(sock.send, self.CLOSE_MESSAGE)
+        with suppress(TLSError, OSError):
             sock.close()
 
-    def test_srv_conf(self, srv_conf, ca1_crt, ee0_crt, ee0_key, trust_store):
+    def test_srv_conf(
+        self,
+        srv_conf,
+        trust_store,
+        srv_psk,
+        ciphers,
+        ca1_crt,
+        ee0_crt,
+        ee0_key,
+    ):
         assert srv_conf.trust_store == trust_store
         assert srv_conf.certificate_chain[0] == (ee0_crt, ca1_crt)
         assert srv_conf.certificate_chain[1] == ee0_key
         assert srv_conf.certificate_chain == ((ee0_crt, ca1_crt), ee0_key)
+        if ciphers:
+            assert srv_conf.ciphers == ciphers
+        assert srv_conf.pre_shared_key_store == srv_psk
 
-    def test_cli_conf(self, cli_conf, trust_store):
+    def test_cli_conf(self, cli_conf, trust_store, cli_psk, ciphers):
         assert cli_conf.trust_store == trust_store
         assert cli_conf.validate_certificates == True
+        if ciphers:
+            assert cli_conf.ciphers == ciphers
+        assert cli_conf.pre_shared_key == cli_psk
 
 
-class TestTLSCommunication(_TestCommunicationBase):
-    proto = socket.SOCK_STREAM
+class _TLSCommunicationBase(_CommunicationBase):
+    @pytest.fixture(scope="class")
+    def proto(self):
+        return socket.SOCK_STREAM
 
     @pytest.fixture(
         scope="class",
@@ -381,51 +423,257 @@ class TestTLSCommunication(_TestCommunicationBase):
 
     @pytest.fixture(scope="class")
     def srv_conf(
-        self, version, ca0_crt, ca1_crt, ee0_crt, ee0_key, trust_store
+        self,
+        version,
+        trust_store,
+        srv_psk,
+        ciphers,
+        ca0_crt,
+        ca1_crt,
+        ee0_crt,
+        ee0_key,
     ):
         return TLSConfiguration(
             trust_store=trust_store,
             certificate_chain=([ee0_crt, ca1_crt], ee0_key),
             lowest_supported_version=TLSVersion.MINIMUM_SUPPORTED,
             highest_supported_version=version,
+            ciphers=ciphers,
+            pre_shared_key_store=srv_psk,
             validate_certificates=False,
         )
 
     @pytest.fixture(scope="class")
-    def cli_conf(self, version, trust_store):
+    def cli_conf(self, version, trust_store, ciphers, cli_psk):
         return TLSConfiguration(
             trust_store=trust_store,
             lowest_supported_version=TLSVersion.MINIMUM_SUPPORTED,
             highest_supported_version=version,
+            ciphers=ciphers,
+            pre_shared_key=cli_psk,
             validate_certificates=True,
         )
 
-    @pytest.fixture(params=[1, 1000, 5000])
+    def echo(self, sock):
+        conn, addr = sock.accept()
+        try:
+            block(conn.do_handshake)
+        except TLSError:
+            conn.close()
+            return
+        while True:
+            data = block(conn.recv, 2 << 13)
+            amt = block(conn.send, data)
+        conn.close()
+
+
+class _DTLSCommunicationBase(_CommunicationBase):
+    @pytest.fixture(scope="class")
+    def proto(self):
+        return socket.SOCK_DGRAM
+
+    @pytest.fixture(scope="class", params=DTLSVersion)
+    def version(self, request):
+        return request.param
+
+    @pytest.fixture(scope="class")
+    def srv_conf(
+        self,
+        version,
+        trust_store,
+        srv_psk,
+        ciphers,
+        ca0_crt,
+        ca1_crt,
+        ee0_crt,
+        ee0_key,
+    ):
+        return DTLSConfiguration(
+            trust_store=trust_store,
+            certificate_chain=([ee0_crt, ca1_crt], ee0_key),
+            lowest_supported_version=DTLSVersion.MINIMUM_SUPPORTED,
+            highest_supported_version=version,
+            ciphers=ciphers,
+            pre_shared_key_store=srv_psk,
+            validate_certificates=False,
+        )
+
+    @pytest.fixture(scope="class")
+    def cli_conf(self, version, trust_store, cli_psk, ciphers):
+        return DTLSConfiguration(
+            trust_store=trust_store,
+            lowest_supported_version=DTLSVersion.MINIMUM_SUPPORTED,
+            highest_supported_version=version,
+            ciphers=ciphers,
+            pre_shared_key=cli_psk,
+            validate_certificates=True,
+        )
+
+    def echo(self, sock):
+        cli, addr = sock.accept()
+        cli.setcookieparam(addr[0].encode("ascii"))
+        with pytest.raises(HelloVerifyRequest):
+            block(cli.do_handshake)
+
+        _, (cli, addr) = cli, cli.accept()
+        _.close()
+        cli.setcookieparam(addr[0].encode("ascii"))
+        try:
+            block(cli.do_handshake)
+        except TLSError:
+            cli.close()
+            return
+        while True:
+            data = block(cli.recv, 4096)
+            # We must use `send()` instead of `sendto()` because the
+            # DTLS socket is connected.
+            amt = block(cli.send, data)
+        cli.close()
+
+
+class TestTLSHostNameVerificationFailure(_TLSCommunicationBase):
+    @pytest.fixture(scope="class")
+    def srv_hostname(self):
+        return "Wrong End Entity"
+
+    @pytest.mark.usefixtures("server")
+    def test_handshake_raises_tlserror(self, client):
+        with pytest.raises(TLSError):
+            client.do_handshake()
+
+
+class TestTLS_PSKAuthentication(_TLSCommunicationBase):
+    @pytest.fixture(scope="class")
+    def ciphers(self):
+        return (
+            "TLS-ECDHE-PSK-WITH-AES-256-CBC-SHA",
+            "TLS-ECDHE-PSK-WITH-AES-128-CBC-SHA",
+            "TLS-DHE-PSK-WITH-AES-256-CBC-SHA",
+            "TLS-DHE-PSK-WITH-AES-128-CBC-SHA",
+            "TLS-RSA-PSK-WITH-AES-256-CBC-SHA",
+            "TLS-RSA-PSK-WITH-AES-128-CBC-SHA",
+            "TLS-PSK-WITH-AES-256-CBC-SHA",
+            "TLS-PSK-WITH-AES-128-CBC-SHA",
+        )
+
+    @pytest.fixture(scope="class")
+    def srv_psk(self):
+        return {"client": b"the secret key"}
+
+    @pytest.fixture(scope="class")
+    def cli_psk(self):
+        return ("client", b"the secret key")
+
+    @pytest.mark.usefixtures("server")
+    def test_handshake_success(self, client):
+        block(client.do_handshake)
+
+
+class TestDTLS_PSKAuthentication(_DTLSCommunicationBase):
+    @pytest.fixture(scope="class")
+    def ciphers(self):
+        return (
+            "TLS-ECDHE-PSK-WITH-AES-256-CBC-SHA",
+            "TLS-ECDHE-PSK-WITH-AES-128-CBC-SHA",
+            "TLS-DHE-PSK-WITH-AES-256-CBC-SHA",
+            "TLS-DHE-PSK-WITH-AES-128-CBC-SHA",
+            "TLS-RSA-PSK-WITH-AES-256-CBC-SHA",
+            "TLS-RSA-PSK-WITH-AES-128-CBC-SHA",
+            "TLS-PSK-WITH-AES-256-CBC-SHA",
+            "TLS-PSK-WITH-AES-128-CBC-SHA",
+        )
+
+    @pytest.fixture(scope="class")
+    def srv_psk(self):
+        return {"client": b"the secret key"}
+
+    @pytest.fixture(scope="class")
+    def cli_psk(self):
+        return ("client", b"the secret key")
+
+    @pytest.mark.usefixtures("server")
+    def test_handshake_success(self, client):
+        block(client.do_handshake)
+
+
+class TestTLS_PSKAuthenticationFailure(_TLSCommunicationBase):
+    @pytest.fixture(scope="class")
+    def ciphers(self):
+        return (
+            "TLS-ECDHE-PSK-WITH-AES-256-CBC-SHA",
+            "TLS-ECDHE-PSK-WITH-AES-128-CBC-SHA",
+            "TLS-DHE-PSK-WITH-AES-256-CBC-SHA",
+            "TLS-DHE-PSK-WITH-AES-128-CBC-SHA",
+            "TLS-RSA-PSK-WITH-AES-256-CBC-SHA",
+            "TLS-RSA-PSK-WITH-AES-128-CBC-SHA",
+            "TLS-PSK-WITH-AES-256-CBC-SHA",
+            "TLS-PSK-WITH-AES-128-CBC-SHA",
+        )
+
+    @pytest.fixture(
+        scope="class",
+        params=[
+            {"client": b"another key"},
+            {"another client": b"the secret key"},
+            {"another client": b"another key"},
+        ],
+    )
+    def srv_psk(self, request):
+        return request.param
+
+    @pytest.fixture(scope="class")
+    def cli_psk(self):
+        return ("client", b"the secret key")
+
+    @pytest.mark.usefixtures("server")
+    def test_handshake_raises_tlserror(self, client):
+        with pytest.raises(TLSError):
+            block(client.do_handshake)
+
+
+class TestDTLS_PSKAuthenticationFailure(_DTLSCommunicationBase):
+    @pytest.fixture(scope="class")
+    def ciphers(self):
+        return (
+            "TLS-ECDHE-PSK-WITH-AES-256-CBC-SHA",
+            "TLS-ECDHE-PSK-WITH-AES-128-CBC-SHA",
+            "TLS-DHE-PSK-WITH-AES-256-CBC-SHA",
+            "TLS-DHE-PSK-WITH-AES-128-CBC-SHA",
+            "TLS-RSA-PSK-WITH-AES-256-CBC-SHA",
+            "TLS-RSA-PSK-WITH-AES-128-CBC-SHA",
+            "TLS-PSK-WITH-AES-256-CBC-SHA",
+            "TLS-PSK-WITH-AES-128-CBC-SHA",
+        )
+
+    @pytest.fixture(
+        scope="class",
+        params=[
+            {"client": b"another key"},
+            {"another client": b"the secret key"},
+            {"another client": b"another key"},
+        ],
+    )
+    def srv_psk(self, request):
+        return request.param
+
+    @pytest.fixture(scope="class")
+    def cli_psk(self):
+        return ("client", b"the secret key")
+
+    @pytest.mark.usefixtures("server")
+    def test_handshake_raises_tlserror(self, client):
+        with pytest.raises(TLSError):
+            block(client.do_handshake)
+
+
+class TestTLSCommunication(_TLSCommunicationBase):
+    @pytest.fixture(params=[100, 1000, 5000])
     def step(self, request):
         return request.param
 
-    def echo(self, sock):
-        conn, addr = sock.accept()
-        block(conn.do_handshake)
-        while True:
-            data = block(conn.recv, 2 << 13)
-            if data == self.CLOSE_MESSAGE:
-                break
-
-            amt = block(conn.send, data)
-            assert amt == len(data)
-
-    def test_server_hostname_fails_verification(self, server, cli_conf):
-        ctx = ClientContext(cli_conf)
-        sock = ctx.wrap_socket(
-            socket.socket(socket.AF_INET, self.proto),
-            server_hostname="Wrong End Entity",
-        )
-        sock.connect(server.getsockname())
-        with pytest.raises(TLSError):
-            block(sock.do_handshake)
-
+    @pytest.mark.usefixtures("server")
     def test_client_server(self, client, buffer, step):
+        block(client.do_handshake)
         received = bytearray()
         for idx in range(0, len(buffer), step):
             view = memoryview(buffer[idx : idx + step])
@@ -434,53 +682,17 @@ class TestTLSCommunication(_TestCommunicationBase):
             assert block(client.recv, 2 << 13) == view
 
 
-class TestDTLSCommunication(_TestCommunicationBase):
-    proto = socket.SOCK_DGRAM
-
-    @pytest.fixture(scope="class", params=DTLSVersion)
-    def version(self, request):
-        return request.param
-
-    @pytest.fixture(scope="class")
-    def srv_conf(
-        self, version, ca0_crt, ca1_crt, ee0_crt, ee0_key, trust_store
-    ):
-        return DTLSConfiguration(
-            trust_store=trust_store,
-            certificate_chain=([ee0_crt, ca1_crt], ee0_key),
-            lowest_supported_version=TLSVersion.MINIMUM_SUPPORTED,
-            highest_supported_version=version,
-            validate_certificates=False,
-        )
-
-    @pytest.fixture(scope="class")
-    def cli_conf(self, version, trust_store):
-        return DTLSConfiguration(
-            trust_store=trust_store,
-            lowest_supported_version=TLSVersion.MINIMUM_SUPPORTED,
-            highest_supported_version=version,
-            validate_certificates=True,
-        )
-
-    @pytest.fixture(params=[10, 1000, 5000])
+class TestDTLSCommunication(_DTLSCommunicationBase):
+    @pytest.fixture(params=[10, 1000])
     def step(self, request):
         return request.param
 
-    def echo(self, sock):
-        cli, addr = sock.accept()
-        cli.setcookieparam(addr[0].encode("ascii"))
-        with pytest.raises(_tls.HelloVerifyRequest):
-            block(cli.do_handshake)
-
-        cli, addr = cli.accept()
-        cli.setcookieparam(addr[0].encode("ascii"))
-        block(cli.do_handshake)
-        while True:
-            data = block(cli.recv, 4096)
-            if data == self.CLOSE_MESSAGE:
-                break
-
-            # We must use `send()` instead of `sendto()` because the
-            # DTLS socket is connected.
-            amt = block(cli.send, data)
-            assert amt == len(data)
+    @pytest.mark.usefixtures("server")
+    def test_client_server(self, client, buffer, step):
+        block(client.do_handshake)
+        received = bytearray()
+        for idx in range(0, len(buffer), step):
+            view = memoryview(buffer[idx : idx + step])
+            amt = block(client.send, view)
+            assert amt == len(view)
+            assert block(client.recv, 2 << 13) == view
